String getRandomWord(byte difficulty) {
  int randElement = random(ARRAY_WORDS_SIZE);
  char buffer[MAX_WORD_SIZE];
  switch (difficulty) {
    case 1:
      strcpy_P(buffer, (char*)pgm_read_word(&(EasyWords[randElement])));
      return buffer;
      break;
    case 2:
      strcpy_P(buffer, (char*)pgm_read_word(&(ModerateWords[randElement])));
      return buffer;
      break;
    case 3:
      strcpy_P(buffer, (char*)pgm_read_word(&(HardWords[randElement])));
      return buffer;
      break;
    default:
      strcpy_P(buffer, (char*)pgm_read_word(&(EasyWords[randElement])));
      return buffer;
      break;
  }
}

void saveSettings(boolean saveScore)
{
  if ((settings.hiscore < currentGame.Score || settings.hiscore == 0) && saveScore == true) {
    settings.hiscore = currentGame.Score;
  }

  settings.difficulty = currentGame.Difficulty;
  eeprom_write_block((const void*)&settings, (void*)0, sizeof(settings));
}


String getValue(String data, char separator, int index) {
  int found = 0;
  int strIndex[] = { 0, -1 };
  int maxIndex = data.length() - 1;

  for (int i = 0; i <= maxIndex && found <= index; i++) {
    if (data.charAt(i) == separator || i == maxIndex) {
      found++;
      strIndex[0] = strIndex[1] + 1;
      strIndex[1] = (i == maxIndex) ? i + 1 : i;
    }
  }
  return found > index ? data.substring(strIndex[0], strIndex[1]) : "";
}

void initializeGame(byte difficulty, byte currentLevel, boolean first) {
  engine.Paused = false;
  engine.PopupDuration = 10;
  gameOverScreen = false;
  switch (difficulty) {
    case 1:
      currentGame.Time = 120;
      currentGame.MaxTime = 120;
      break;
    case 2:
      currentGame.Time = 100;
      currentGame.MaxTime = 100;
      break;
    case 3:
      currentGame.Time = 70;
      currentGame.MaxTime = 70;
      break;
    default:
      currentGame.Time = 120;
      currentGame.MaxTime = 120;
      break;
  }
  String notParsedWord = getRandomWord(difficulty);
  currentGame.CurrentWord = getValue(notParsedWord, ';', 0);
  char temp[MAX_CATEGORY_SIZE];
  strcpy_P(temp, (char*)pgm_read_word(&(Categories[getValue(notParsedWord, ';', 1).toInt()])));
  currentGame.Category = String(temp);
  currentGame.CurrentWordLength = currentGame.CurrentWord.length();
  currentGame.CurrentLevel = currentLevel;
  currentGame.WrongAnswers = 0;

  if (first) {
    currentGame.Score = 0;
  }

  memset(guessedLetters, 0, sizeof(guessedLetters));
  memset(missedLetters, 0, sizeof(missedLetters));
  memset(lastGuess, 0, sizeof(lastGuess));
}

void drawHangman() {

  if (currentGame.WrongAnswers >= 1) {
    gb.display.drawLine(45, 31, 48, 27);
  }
  if (currentGame.WrongAnswers >= 2) {
    gb.display.drawLine(48, 27, 50, 31);
  }
  if (currentGame.WrongAnswers >= 3) {
    gb.display.drawLine(48, 27, 48, 9);
  }
  if (currentGame.WrongAnswers >= 4) {
    gb.display.drawLine(48, 9, 58, 9);
  }
  if (currentGame.WrongAnswers >= 5) {
    gb.display.drawLine(58, 9, 58, 13);
  }
  if (currentGame.WrongAnswers >= 6) {
    gb.display.drawCircle(58, 16, 2);
  }
  if (currentGame.WrongAnswers >= 7) {
    gb.display.drawLine(58, 19, 58, 22);
  }
  if (currentGame.WrongAnswers >= 8) {
    gb.display.drawLine(57, 23, 57, 26);
    gb.display.drawLine(59, 23, 59, 26);
  }
  if (currentGame.WrongAnswers >= 9) {
    gb.display.drawPixel(55, 21);
    gb.display.drawLine(56, 20, 60, 20);
    gb.display.drawPixel(61, 21);
  }
}

bool searchIfGuessedAlready() {
  char *s;
  s = strstr(guessedLetters, lastGuess);

  if (s != NULL) {
    return false;
  } else {
    return true;
  }
}

bool searchIfMissedAlready() {
  char *s;
  s = strstr(missedLetters, lastGuess);

  if (s != NULL) {
    return false;
  } else {
    return true;
  }
}

bool checkIfGuessedAll() {
  char *s;
  byte guessedCount = 0;
  String guessed(guessedLetters);
  for (int i = 0; i < currentGame.CurrentWordLength; i++) {
    if (guessed.indexOf(currentGame.CurrentWord.charAt(i)) > -1) {
      guessedCount += 1;
    }
  }

  if (guessedCount == currentGame.CurrentWordLength) {
    return true;
  } else {
    return false;
  }
}

bool checkValidChar() {
  bool valid;
  for (int i = 0; i < ARRAY_LETTERS_SIZE; i++) {
    if (letters[i] == lastGuess[0]) {
      valid = true;
      break;
    } else {
      valid = false;
    }
  }
  return valid;
}

unsigned int calculateScore() {
  int score = (currentGame.Time * currentGame.Difficulty) + (currentGame.CurrentWordLength * currentGame.Difficulty) + currentGame.Time;
  if (score > 0) {
    return score;
  } else {
    return 0;
  }
}

void game_loop() {
  while (1) {
    if (gb.update()) {
      if (gb.buttons.pressed(BTN_C)) {
        gb.display.setFont(font5x7);
        gb.titleScreen(logo);
        gb.pickRandomSeed();
        gb.battery.show = false;
        engine.Paused = true;
      }

      if (gb.buttons.pressed(BTN_A)) {
        engine.Paused = false;
        gb.display.setFont(font3x5);
        gb.keyboard(lastGuess, 2);

        if (currentGame.CurrentWord.indexOf(lastGuess) > -1 && searchIfGuessedAlready()) {
          strcat(guessedLetters, lastGuess);
          gb.sound.playOK();
          if (checkIfGuessedAll()) {
            currentGame.Score += calculateScore();
            gameOver(true);
            initializeGame(currentGame.Difficulty, currentGame.CurrentLevel += 1, false);
          } else {
            gb.popup(F("Letter found!"), engine.PopupDuration);
          }
        } else {
          if (checkValidChar()) {
            if (searchIfMissedAlready()) {
              gb.sound.playCancel();
              gb.popup(F("Letter not found!"), engine.PopupDuration);
              strcat(missedLetters, lastGuess);
              currentGame.WrongAnswers += 1;
            } else {
              gb.popup(F("Letter repeat!"), engine.PopupDuration);
            }
          } else {
            gb.popup(F("Invalid character!"), engine.PopupDuration);
          }
        }
      }

      if (gb.buttons.pressed(BTN_B)) {
        displayDebug();
      }

      if ((gb.frameCount - engine.Frames) >= 20 && !engine.Paused) {
        engine.Frames = gb.frameCount;
        currentGame.Time --;
      }

      gb.display.setFont(font3x3);

      gb.display.cursorX = 1;
      gb.display.cursorY = 1;
      gb.display.print(F("Time: "));
      gb.display.print(currentGame.Time);

      gb.display.cursorX = 1;
      gb.display.cursorY = 5;
      gb.display.print(F("Score:"));
      gb.display.print(currentGame.Score);

      if (currentGame.WrongAnswers >= 1) {
        gb.display.cursorX = 46;
        gb.display.cursorY = 1;
        gb.display.print(currentGame.WrongAnswers);
        gb.display.print(F("/10"));
      }

      if (currentGame.Difficulty == 1) {
        gb.display.cursorX = 1;
        gb.display.cursorY = 9;
        gb.display.print(currentGame.Category);
      }

      gb.display.setFont(font3x5);
      gb.display.cursorX = 1;

      drawHangman();

      for (int i = 0; i < currentGame.CurrentWordLength; i++) {
        gb.display.cursorY = 35;

        for (int j = 0; j < 26; j++) {
          if (guessedLetters[j] == currentGame.CurrentWord[i] ) {
            gb.display.drawChar(gb.display.cursorX, gb.display.cursorY, currentGame.CurrentWord[i], 1);
          }
        }

        gb.display.cursorX += 5;
      }

      gb.display.cursorX = 1;

      for (int i = 0; i < currentGame.CurrentWordLength; i++) {
        gb.display.cursorY = 40;
        gb.display.print(F("_"));
        gb.display.cursorX += 1;
      }
    }

    if (currentGame.WrongAnswers == 9) {
      gb.display.cursorX = 1;
      gb.display.cursorY = 18;
      gb.display.print(F("Last guess!"));
    }

    if (missedLetters[0] == '\0') {
      gb.display.cursorX = 1;
      gb.display.cursorY = 22;

      gb.display.print(F("Press"));
      gb.display.setFont(font5x7);
      gb.display.print(F(" \25 "));
      gb.display.setFont(font3x5);
      gb.display.print(F("to guess"));
    }

    if ((currentGame.Time <= 0 || currentGame.WrongAnswers >= 10) && !gameOverScreen) {
      if (currentGame.CurrentLevel == 1) {
        currentGame.Score = 0;
      } else {
        currentGame.Score += calculateScore();
      }
      gameOver(false);
      return;
    }
  }
}
