#include <SPI.h>
#include <Gamebuino.h>
#include <avr/eeprom.h>
#include <string.h>
#include <avr/pgmspace.h>
Gamebuino gb;

#define MENULENGTH 5
#define MENUDIFLENGTH 3
#define ARRAY_WORDS_SIZE 34
#define ARRAY_CATEGORIES_SIZE 20
#define ARRAY_LETTERS_SIZE 26
#define MAX_WORD_SIZE 15
#define MAX_CATEGORY_SIZE 10
#define VERSION 0.1
#define DEBUG 0

const char strNewGame[] PROGMEM = "New game";
const char strDifficulty[] PROGMEM = "Difficulty";
const char strScores[] PROGMEM = "High score";
const char strAbout[] PROGMEM = "About";
const char strSystemInfo[] PROGMEM = "System info";

const char strEasy[] PROGMEM = "Easy";
const char strMedium[] PROGMEM = "Medium";
const char strHard[] PROGMEM = "Hard";

extern const byte PROGMEM logo[];

const char* const menu[MENULENGTH] PROGMEM = {
  strNewGame,
  strDifficulty,
  strScores,
  strAbout,
  strSystemInfo,
};

const char* const menuDifficulty[MENULENGTH] PROGMEM = {
  strEasy,
  strMedium,
  strHard,
};

typedef struct Game {
  byte Difficulty = 1;
  byte Time;
  byte MaxTime;
  byte CurrentLevel;
  String CurrentWord;
  String Category;
  byte CurrentWordLength;
  byte WrongAnswers;
  unsigned int Score;
};

typedef struct Engine {
  int Frames;
  boolean Paused;
  byte PopupDuration;
};

struct settings_t
{
  unsigned int hiscore;
  byte difficulty;
} settings;

Game currentGame;
Engine engine;

extern const byte font3x3[];
extern const byte font3x5[];
extern const byte font5x7[];

char lastGuess[2];
char guessedLetters[26];
char missedLetters[26];
boolean gameOverScreen = false;

void setup() {
  eeprom_read_block((void*)&settings, (void*)0, sizeof(settings));

  if (settings.hiscore == 65535) {
    settings.hiscore = 0;
  }

  if (settings.difficulty == 255 || settings.difficulty == 0) {
    settings.difficulty = 1;
    currentGame.Difficulty = 1;
  } else {
    currentGame.Difficulty = settings.difficulty;
  }
  
  gb.begin();
  gb.display.setFont(font5x7);
  gb.titleScreen(logo);
  gb.pickRandomSeed();
  gb.battery.show = false;
}

void loop() {
  switch (gb.menu(menu, MENULENGTH)) {
    case -1: //nothing selected
      gb.titleScreen(logo);
      gb.pickRandomSeed();
      gb.battery.show = false;
      break;
    case 0: //new game
      initializeGame(currentGame.Difficulty,1,true);
      game_loop();
      break;
    case 1: //options
      displayDifficulty();
      break;
    case 2: //high scores
      displayScores();
      break;      
    case 3: //about
      displayAbout();
      break;
    case 4: //system info
      displaySystemInfo();
      break;
    default:
      break;
  }
}
